@@ -729,6 +729,8 @@ int AsyncSocket_SendWithFd(AsyncSocket *asock, void *buf, int len, int passFd,
 int AsyncSocket_IsSendBufferFull(AsyncSocket *asock);
 int AsyncSocket_GetNetworkStats(AsyncSocket *asock,
                                 AsyncSocketNetworkStats *stats);
+int AsyncSocket_GetSNIHostname(AsyncSocket *asock,
+                               const char **sniHostname);
 int AsyncSocket_CancelRecv(AsyncSocket *asock, int *partialRecvd, void **recvBuf,
                            void **recvFn);
 int AsyncSocket_CancelRecvEx(AsyncSocket *asock, int *partialRecvd, void **recvBuf,
@@ -840,24 +842,7 @@ void AsyncSocket_WebSocketServerSendError(AsyncSocket *asock, const char *text);
  */
 #define ASOCKPREFIX "SOCKET "
 
-/* gcc needs special syntax to handle zero-length variadic arguments */
-#if defined(_MSC_VER)
-#define ASOCKWARN(_asock, fmt, ...)                               \
-   Warning(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
-           AsyncSocket_GetFd(_asock), __VA_ARGS__)
-
-#define ASOCKLG0(_asock, fmt, ...)                            \
-   Log(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
-       AsyncSocket_GetFd(_asock), __VA_ARGS__)
-
-#define ASOCKLOG(_level, _asock, fmt, ...)                          \
-   do {                                                             \
-      if (((_level) == 0) || DOLOG_BYNAME(asyncsocket, (_level))) { \
-         Log(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
-             AsyncSocket_GetFd(_asock), __VA_ARGS__);               \
-      }                                                             \
-   } while (0)
-#else
+/* Both gcc and msvc support ##__VA_ARGS__ to handle zero-length variadic arguments */
 #define ASOCKWARN(_asock, fmt, ...)                               \
    Warning(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
            AsyncSocket_GetFd(_asock), ##__VA_ARGS__)
@@ -873,7 +858,6 @@ void AsyncSocket_WebSocketServerSendError(AsyncSocket *asock, const char *text);
              AsyncSocket_GetFd(_asock), ##__VA_ARGS__);             \
       }                                                             \
    } while (0)
-#endif
 
 #if defined(__cplusplus)
 }  // extern "C"
