@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2004-2017 VMware, Inc. All rights reserved.
+ * Copyright (C) 2004-2020 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -235,6 +235,8 @@ enum {
    VIX_E_HOST_TCP_CONN_LOST                     = 14011,
    VIX_E_HOST_NBD_HASHFILE_VOLUME               = 14012,
    VIX_E_HOST_NBD_HASHFILE_INIT                 = 14013,
+   VIX_E_HOST_SERVER_SHUTDOWN                   = 14014,
+   VIX_E_HOST_SERVER_NOT_AVAILABLE              = 14015,
    
    /* Disklib errors */
    VIX_E_DISK_INVAL                             = 16000,
