@@ -1824,9 +1824,16 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE1_V_MNR  0
 #define   TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE1_V_BASE 5
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE1
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE1)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE1)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2_V_MJR  13
+#define   TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2_V_MNR  0
+#define   TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2_V_BASE 10
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_SWISS_ARMY_KNIFE_UPDATE2)
 
 /*
  * The extended Tools version is the current Tools version with the
